#
#               Common Quantification
#                        of
#              Categorical Variables
#
#             Yasuharu Okamoto, 2016.09, 2020.09
#
import math
import numpy
import scipy.stats as ss
import pandas as pd      #   xlrd and openpyxl are needed 
from readdata import *
import cmmnq

def ToFloat(X):
    FX = []
    for v in X:
        FX.append(float(v))
    return FX

f_out, s_out, n_data, n_vars, var_names, var_cond, X = ReadExcelFile()

#
#       Xgen:  a matrix of scores, which are the same as the input value for condition = 0,
#                                                standardized scores for conditon = 1,
#                                                or quantifications for conditon >= 2.
#
Xgen = []
pos_v = 0
for i_var in range(0, n_vars):
    if (var_cond[i_var] < 0):                           #  Invalid condition value
        print("var_cond < 0")
        raise "var_cond error."
    else:        
        if (var_cond[i_var] == 0):                     #  No transformation
            Xgen.append([var_names[i_var], X[i_var]])
            pos_v = pos_v + 1
        elif (var_cond[i_var] == 1):                   #  Standardization
            vX = ss.zscore(ToFloat(X[i_var]))
            Xgen.append([var_names[i_var] + '_z', vX])
            pos_v = pos_v + 1
        else:                                           #  Quantification
            vX = X[i_var]
            for iv in range(0, n_data):
                vX[iv] = int(vX[iv])
            n_quant, eigen_val, matOmega, z = cmmnq.comq(vX, var_cond[i_var], n_data)
            f_out.write("\n")
            f_out.write("Categorical variable: " + var_names[i_var]
                                                 + " quantified.")
            f_out.write("Number of quantification variables = %d\n"%n_quant)
            for i in range(0, n_quant):
                f_out.write("\nEigen value = %-10.5f\n"%eigen_val[i])
                f_out.write("Eigen vector =\n")
                for j in range(0, var_cond[i_var]):
                    f_out.write("%20.5f\n"%matOmega[j,i])
            f_out.write("\n")
            for iq in range(0, n_quant):
                Xgen.append(['{0}-{1}'.format(var_names[i_var], iq), z.transpose()[iq]])
                pos_v = pos_v + 1

#
#       Writing the contents of Xgen to the output file
#
f_out.write("")
f_out.write("Results...\n\n")
for i in Xgen:
    f_out.write(mm.get_str(i[0]))
f_out.write("\n")
for i in range(n_data):
    for ix in Xgen:
        f_out.write(mm.get_str(ix[1][i]))
    f_out.write("\n")
          
#
#           Output to a Excel file
#
data_t = {}
for i in range(len(Xgen)):
    data_t[Xgen[i][0]] = Xgen[i][1]
xl_outnm = input('Excel file name for output (*+.xlsx) = ')
df_out = pd.DataFrame(data_t)
writer = pd.ExcelWriter(xl_outnm)
df_out.to_excel(writer, index = False)
writer.save()

print('\n' + xl_outnm, 'was saved.')

f_out.close()

print('\n' + s_out, 'was saved.')
