import numpy as np
import math

def eigen_sym(A):
    """
            Re-arranged results from numpy.linalg.eigh
            The eigen values and vectors are re-ordered
            according to descending order of absolute values
            of eigen values
            Returned values:
                w : eigen values
                vrow : eigen vectors, each vector as a row vector 
            
    """
    p = A.shape[0]
    w = None
    vrow = None
    if p == 1:
        w = np.full((1,),A[0][0])
        vrow = np.full((1,1), 1)
    else:
        w, v = np.linalg.eigh(A)
        vtp = v.transpose()
        w_vtp = []
        for i in range(p):
            w_vtp.append([w[i], vtp[i]])
        #
        #       Arranging according to
        #       absolute values of eigen values
        #
        w_vtp.sort(key = lambda x: -math.fabs(x[0]))
        w = []
        vrow = []
        for x in w_vtp:
            w.append(x[0])
            vrow.append(x[1])
        #
        #   Transforming List arrays to ndarrays
        #
        w = np.array(w)
        vrow = np.array(vrow)
 
        return w, vrow

